--[[
AuraToChat, World of Warcraft addon that output buff gains to the in-game
ChatFrame.
Copyright (C) 2007 Rabbit

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

local RL = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Tweak these in order to change the colors used.
-- Colors are generated by changing the HSL hue by around 30 for each spell.
local defaultR = 255
local defaultG = 204
local defaultB = 153

-----------------------------------------------------------------------
-- Locale

local PRINT_STRING = "%s on %s."
local ADD = "Add"
local ADD_DESC = "Adds a buff or debuff."
local REMOVE = "Remove"
local REMOVE_DESC = "Removes a buff or debuff."
local LIST = "List"
local LIST_DESC = "Lists the registered auras."
local AURA_ADDED = "%s added."
local AURA_REMOVED = "%s removed."
local USAGE = "<aura name>"
local FIRST_USE = "AuraToChat has not been configured yet, you need to add auras to watch with the /atc add command."
do
	local locale = GetLocale()
	if locale == "koKR" then
		-- etc
	end
end

-----------------------------------------------------------------------
-- Utility functions and tables

-- Keyed table, key = spell name, value = color
local spellToColoredSpell = {}

local classColors = setmetatable({}, {__index =
	function(self, key)
		local r, g, b = 0.63, 0.63, 0.63
		if RAID_CLASS_COLORS and RAID_CLASS_COLORS[key] then
			r, g, b = RAID_CLASS_COLORS[key].r, RAID_CLASS_COLORS[key].g, RAID_CLASS_COLORS[key].b
		end
		self[key] = "|cff" .. string.format("%02x%02x%02x", r * 255, g * 255, b * 255)
		return self[key]
	end
})
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = RL.roster[key] and RL.roster[key].class or "WTF"
		self[key] = classColors[class] .. key .. "|r"
		return self[key]
	end
})

local function findRGB(q, q2, hue)
	if hue > 360 then hue = hue - 360 end
	if hue < 0 then hue = hue + 360 end

	if hue < 60 then return q + (q2 - q) * hue / 60
	elseif hue < 180 then return q2
	elseif hue < 240 then return q + (q2 - q) * (240 - hue) / 60
	else return q
	end
end

local function hslToRGB(h, s, l)
	local l2, s2 = l / 100, s / 100

	local p2 = l2 + s2 - (l2 * s2)
	if l2 <= 0.5 then p2 = l2 * (1 + s2) end

	local p1 = 2 * l2 - p2

	local r, g, b = l2, l2, l2
	if s2 ~= 0 then
		r = findRGB(p1, p2, h + 120)
		g = findRGB(p1, p2, h)
		b = findRGB(p1, p2, h - 120)
	end
	r = math.ceil(r * 255)
	g = math.ceil(g * 255)
	b = math.ceil(b * 255)
	return r, g, b
end

local function rgbToHSL(r, g, b)
	local r2, g2, b2 = r / 51 * 0.2, g / 51 * 0.2, b / 51 * 0.2
	local max, min = 0, 0
	if r2 >= g2 then max = r2
	else max = g2 end
	if b2 > max then max = b2 end
	if r2 <= g2 then min = r2
	else min = g2 end
	if b2 < min then min = b2 end
	local l = (max + min) / 2
	local s, h = 0, 0
	if max ~= min then
		if l < .5 then
			s = (max - min) / (max + min)
		else
			s = (max - min) / (2 - max - min)
		end
		if r2 == max then h = (g2 - b2) / (max - min) end
		if g2 == max then h = (2 + (g2 - r2)) / (max - min) end
		if b2 == max then h = (4 + (r2 - g2)) / (max - min) end
	end
	h = math.ceil(h * 60)
	if h < 0 then h = h + 360
	elseif h >= 360 then h = h - 360 end
	s = math.ceil(s * 100)
	l = math.ceil(l * 100)
	return h, s, l
end

local function generateColors()
	for k in pairs(spellToColoredSpell) do
		spellToColoredSpell[k] = nil
	end
	local h, s, l = rgbToHSL(defaultR, defaultG, defaultB)
	for i = 1, #AuraToChatDB.Auras do
		local r, g, b = hslToRGB(h, s, l)
		spellToColoredSpell[AuraToChatDB.Auras[i]] = "|cff" .. string.format("%02x%02x%02x", r, g, b) .. AuraToChatDB.Auras[i] .. "|r"
		h = h + 30
		if h > 360 then h = h - 360 end
	end
end

-----------------------------------------------------------------------
-- Addon

AuraToChat = AceLibrary("AceAddon-2.0"):new(
	"AceEvent-2.0",
	"AceConsole-2.0"
)
local addon = AuraToChat

function addon:OnInitialize()
	if type(AuraToChatDB) ~= "table" then
		AuraToChatDB = {} -- Not used yet
	end
	if type(AuraToChatDB.Auras) ~= "table" then
		AuraToChatDB.Auras = {}
	end

	self:RegisterChatCommand("/atc", {
		type = "group",
		args = {
			add = {
				type = "text",
				name = ADD,
				desc = ADD_DESC,
				get = false,
				set = function(aura)
					table.insert(AuraToChatDB.Auras, aura)
					self:Print(AURA_ADDED:format(aura))
					generateColors()
				end,
				usage = USAGE,
			},
			remove = {
				type = "text",
				name = REMOVE,
				desc = REMOVE_DESC,
				get = false,
				set = function(aura)
					for i, v in ipairs(AuraToChatDB.Auras) do
						if v == aura then
							local x = table.remove(AuraToChatDB.Auras, i)
							self:Print(AURA_REMOVED:format(x))
							break
						end
					end
				end,
				usage = USAGE,
			},
			list = {
				type = "execute",
				name = LIST,
				desc = LIST_DESC,
				func = function()
					for i, v in ipairs(AuraToChatDB.Auras) do
						self:Print(v)
					end
				end,
			},
		},
	}, "AURATOCHAT")

end

function addon:OnEnable(first)
	if #AuraToChatDB.Auras == 0 then
		self:Print(FIRST_USE)
	elseif first then
		generateColors()
	end

	self:RegisterEvent("SpecialEvents_UnitBuffGained", "Aura")
	self:RegisterEvent("SpecialEvents_UnitDebuffGained", "Aura")
end

function addon:Aura(unit, name)
	if not spellToColoredSpell[name] then return end
	local n = UnitName(unit)
	if not n or not RL.roster[n] then return end
	self:Print(PRINT_STRING:format(spellToColoredSpell[name], coloredName[n]))
end

function addon:Test()
	for k, v in pairs(spellToColoredSpell) do
		self:Aura("player", k)
	end
end

